#version 120

#define waterRefraction
#define waterSpecular
#define waterSkyReflections
#define refractionStrength 1.0 //[0.1 0.2 0.3 0.4 0.5 0.6 0.7 0.8 0.9 1.0 1.1 1.2 1.3 1.4 1.5 1.6 1.7 1.8 1.9 2.0]

uniform sampler2D colortex0;
uniform sampler2D colortex1;
uniform sampler2D colortex2;
uniform sampler2D colortex4;

uniform vec3 sunPosition;
uniform vec3 moonPosition;
uniform vec3 upPosition;
uniform vec3 shadowLightPosition;

uniform ivec2 eyeBrightnessSmooth;


uniform int isEyeInWater;

// uniform float viewHeight;
// uniform float viewWidth;

// uniform int frameCounter;

varying vec2 texcoord;

#include "/lab/common.glsl"
#include "/lab/conversions.glsl"
#include "/lab/sample_depth.glsl"
#include "/lab/samplers.glsl"
// #include "/lab/rt.glsl"
// #include "/lab/dither.glsl"
#include "/lab/time.glsl"
#include "/lab/sky.glsl"
// #include "/lab/fog.glsl"

// vec3 coloredShadowsCalc(vec3 color, vec3 screenCoord, vec3 viewCoord)
// {
// 	vec3 coloredShads = vec3(1.0);
// 	bool isShadow = false;
// 	vec3 shadowVec = screenCoord;
// 	const int sampCount = shadowSamples;

// 	float dither = bayer32(gl_FragCoord.xy);

// 	marchTheDamnRay(false, false, 0.2, 1.6, sampCount, viewCoord, dither, normalize(shadowLightPosition), shadowVec, isShadow, coloredShads);
	
// 	float shadowID = texture(colortex1, shadowVec.xy).b;

// 	return mix(color, textureLod(colortex0, shadowVec.xy, 14.0).rgb, float(isShadow)*float(shadowID == 1191.0));

// }

vec3 fastRefract(vec3 color, vec3 viewCoord, vec3 normalData)
{
	vec3 direction = refract(normalize(viewCoord), pow(normalData, vec3(1.0 / 50.0)) * refractionStrength, 0.75);
	vec3 sampCoord = viewToScreen(viewCoord + direction);
	if(clamp01(sampCoord) == sampCoord) return texture(colortex0, sampCoord.xy).rgb;
	return color;
}

vec3 fastSkyReflections(vec3 color, vec3 viewCoord, vec3 normalData, vec3 screenCoord, float brightnessMult)
{
	vec3 direction = reflect(normalize(viewCoord), normalData);
	vec3 eyeReflectedPos = viewToEye(direction);
	vec3 eyePos = viewToEye(viewCoord);
	vec3 skyData = skyCol(eyeReflectedPos);

	float mixFactor = max0(getAngle(dot(normalize(upPosition), direction)));
	//float sampledSkylight = texture2D(colortex4, (viewToScreen(viewCoord + direction) - screenCoord).xy).a;

	return mix(color, skyData, mixFactor * (1.0 - clamp01(pow2(length(eyePos.xz)/far))) * brightnessMult);
	
}

vec3 fastSpecular(vec3 color, vec3 viewCoord, vec3 normalData, float brightnessMult)
{
	vec3 direction = reflect(normalize(viewCoord), normalData);
	vec3 sampCoord = viewToEye(direction);
	float specularOverall = smoothstep(0.075, 0.0096, distance(sampCoord, viewToEye(normalize(sunPosition))));
	specularOverall += smoothstep(0.075, 0.0096, distance(sampCoord, viewToEye(normalize(moonPosition))));

	return color + specularOverall * pow2(shortTex(colortex4).a) * brightnessMult;
}

void main() {
	vec3 color = texture2D(colortex0, texcoord).rgb;
	vec3 screenData = vec3(texcoord, depth1);
	// vec3 screenData0 = vec3(texcoord, depth0);
	vec3 viewData = screenToView(screenData);
	vec3 normalData = shortTex(colortex2).rgb * 2.0 - 1.0;
	int idData = int(shorterTexFetch(colortex1).b * 65535);

	// #ifdef coloredShadows
	// color = coloredShadowsCalc(color, screenData, screenToView(screenData));
	// #endif

	if(idData == 8 || idData == 9)
	{
		float brightnessMult = pow2(eyeBrightnessSmooth.y*rcp(240.0));

		#ifdef waterRefraction
		color = fastRefract(color, viewData, normalData);
		#endif

		#ifdef waterSpecular
		if(isEyeInWater == 0) color = fastSpecular(color, viewData, normalData, brightnessMult);
		#endif
		
		#ifdef waterSkyReflections
		if(isEyeInWater == 0) color = fastSkyReflections(color, viewData, normalData, screenData, brightnessMult);
		#endif
	}

	// if(abs(depth0-depth1) > 0.0)
	// {
	// 	vec3 eyePos = screenToEye(screenData0);
	// 	vec3 skyData = skyCol(normalize(eyePos));
	// 	color = fogMix(color, skyData, eyePos);
	// }


/* DRAWBUFFERS:0 */
	gl_FragData[0] = vec4(color, 1.0); //colortex0
}