#version 150 compatibility

#define customWaterColorR 0.4 //[0.1 0.2 0.3 0.4 0.5 0.6 0.7 0.8 0.9 1.0]
#define customWaterColorG 0.6 //[0.1 0.2 0.3 0.4 0.5 0.6 0.7 0.8 0.9 1.0]
#define customWaterColorB 1.0 //[0.1 0.2 0.3 0.4 0.5 0.6 0.7 0.8 0.9 1.0]

#define lightSaturation 2.0 //[0.1 0.2 0.3 0.4 0.5 0.6 0.7 0.8 0.9 1.0 1.1 1.2 1.3 1.4 1.5 1.6 1.7 1.8 1.9 2.0 2.1 2.2 2.3 2.4 2.5]
#define SSShadows
#define shadowSamples 64 //[8 16 32 64 128 256]
const float sunPathRotation = -25.0f;

uniform sampler2D colortex0;
uniform sampler2D colortex1;
uniform sampler2D colortex2;
uniform sampler2D colortex3;
uniform sampler2D colortex4;

uniform int frameCounter;
uniform int isEyeInWater;

uniform float viewWidth;
uniform float viewHeight;
//uniform float far;

uniform vec3 shadowLightPosition;

varying vec2 texcoord;

#include "/lab/common.glsl"
#include "/lab/samplers.glsl"
#include "/lab/conversions.glsl"
#include "/lab/bufferinfo.glsl"
#include "/lab/dither.glsl"
#include "/lab/sample_depth.glsl"
#include "/lab/rt.glsl"
#include "/lab/blur.glsl"
#include "/lab/color.glsl"
#include "/lab/time.glsl"
#include "/lab/sky.glsl"
#include "/lab/fog.glsl"

vec3 applyLighting(vec3 color, vec3 normals, vec3 viewCoord, vec3 screenCoord, vec3 flatNorms)
{
	float lambertDiffuse = (max0(dot(normals, normalize(shadowLightPosition))));
	vec4 coloredShads = vec4(1.0);
	bool isShadow = false;
	#ifdef SSShadows
	vec3 shadowVec = viewToScreen(viewCoord + flatNorms * 1e-4);
	const int sampCount = shadowSamples;
	#endif 

	float dither = bayer32(gl_FragCoord.xy);

	#ifdef SSShadows
	marchTheDamnRay(false, false, 0.2, 0.1, sampCount, viewCoord, dither, normalize(shadowLightPosition), shadowVec, isShadow, coloredShads);
	#endif

	if(coloredShads.a < 0.2) isShadow = false;

	float outlineAO = depthBlur(colortex3, 3, depth0, depthtex0);

	vec4 lmapData = shortTex(colortex4);
	lmapData.rgb = rgb2hsv(lmapData.rgb);
	
	vec4 ctex1Data = shortTex(colortex1);

	float torchLightValue = ctex1Data.a;
	vec3 torchLight = hsv2rgb(vec3(lmapData.r,lmapData.g*lightSaturation, rgb2hsv(vec3(torchLightValue)).b))*torchLightValue;

	float skyLight = lmapData.a;
	int blockid = int(ctex1Data.b * 65535);
	if(blockid == 1194) lambertDiffuse = 1.0;
	float shadowData = min(float(!isShadow), lambertDiffuse);
	// if(blockid == 1194.0) shadowData = smoothstep(0.0, 1.0, shadowData);
	vec3 overallShadows = mix(vec3(0.25,0.35,0.5), byTime(vec3(1.0, 1.0, 0.4), vec3(1.0), vec3(1.0, 1.0, 0.4), vec3(1.0)), byTime(1.0, 1.0, 1.0, 0.05) * shadowData * pow((skyLight), 5.0));
	vec3 overallMult = (overallShadows * mix(skyLight, skyLight*byTime(1.0, 1.0, 1.0, 0.2), skyLight)  + torchLight) * outlineAO;
	return color * overallMult;
}

void main() {
	vec3 color = shortTex(colortex0).rgb;

	// vec3 normals = unpackNormalData(data1.r);

	vec3 normals = shortTex(colortex2).rgb * 2.0 - 1.0; 

	vec3 flatNormals = shortTex(colortex1).rgb * 2.0 - 1.0;

	flatNormals = normalZRecontrust(flatNormals);

	vec3 screenPos = vec3(texcoord, depth0);

	vec3 eyePos = screenToEye(screenPos);

	if(isTerrain) {
		color = applyLighting(color, normals, screenToView(screenPos), screenPos, flatNormals);
		vec3 skyData = skyCol(normalize(eyePos));
		color = fogMix(color, skyData, eyePos);
	}	

	if(isEyeInWater == 1) color = fogMix(color, pow(vec3(customWaterColorR, customWaterColorG, customWaterColorB), vec3(4.0)), 5.5*(eyePos));

	// color *= round(max0(dot(normals, normalize(shadowLightPosition))) * 8.0) * rcp(8.0);


/* DRAWBUFFERS:0 */
	gl_FragData[0] = vec4(color, 1.0); //gcolor
}