#version 150 compatibility

uniform sampler2D lightmap;
uniform sampler2D normals;
uniform sampler2D specular;
uniform sampler2D texture;

#ifdef entities
uniform int entityId;
#endif

flat in float blockid;
in vec2 lmcoord;
in vec2 texcoord;
flat in vec3 normal;
in vec4 glcolor;
flat in mat3 tbn;

#include "/lab/common.glsl"
#include "/lab/gamma.glsl"
#include "/lab/bufferinfo.glsl"

void main() {
	vec2 coord = texcoord;

	vec4 color = texture2D(texture, coord) * vec4(glcolor.rgb, 1.0);

	#ifdef entities
	if(color.a < 0.15) {
		discard;
		return;
	}
	#ifdef IS_IRIS 
	if(entityId == 422) {
		discard;
		return;
	}
	#endif
	#endif

	color.rgb = sRGBtoLinear(color.rgb);

	vec4 normalMap = texture2D(normals, coord) * 2.0 - 1.0;

	float normalAO = normalMap.z;

    normalMap.rgb = normalZRecontrust(normalMap.rgb);

	normalMap.rgb = tbn*normalMap.rgb;


/* DRAWBUFFERS:0124 */
	gl_FragData[0] = color; //gcolor
	gl_FragData[1] = vec4(normal.xy * 0.5 + 0.5, blockid * rcp(65535.0), lmcoord.x);
	gl_FragData[2] = vec4(normalMap.rgb * 0.5 + 0.5, normalAO);
	gl_FragData[3] = vec4(texture(lightmap, lmcoord.xy).rgb, lmcoord.y);
}