#version 120

uniform sampler2D lightmap;
uniform sampler2D normals;
uniform sampler2D specular;
uniform sampler2D texture;

uniform float far;

varying float blockid;
varying vec2 lmcoord;
varying vec2 texcoord;
varying vec3 normal;
varying vec3 eyePos;
varying vec4 glcolor;
varying mat3 tbn;

#include "/lab/common.glsl"
#include "/lab/gamma.glsl"
#include "/lab/bufferinfo.glsl"
#include "/lab/time.glsl"
#include "/lab/sky.glsl"
#include "/lab/fog.glsl"

void main() {
	vec2 coord = texcoord;

	int blockidINT = int(floor(blockid));

	vec4 color = texture2D(texture, coord) * vec4(glcolor.rgb, 1.0);

	color.rgb = sRGBtoLinear(color.rgb);

	vec4 normalMap = texture2D(normals, coord) * 2.0 - 1.0;

	float normalAO = normalMap.z;

    normalMap.rgb = normalZRecontrust(normalMap.rgb);

	normalMap.rgb = tbn*normalMap.rgb;

	vec3 skyData = skyCol(normalize(eyePos));

	color.rgb = fogMix(color.rgb, skyData, eyePos);
    
/* DRAWBUFFERS:0124 */
	gl_FragData[0] = color; //gcolor
	gl_FragData[1] = vec4(normal.xy * 0.5 + 0.5, blockidINT, lmcoord.x);
	gl_FragData[2] = vec4(normalMap.rgb * 0.5 + 0.5, normalAO);
	gl_FragData[3] = vec4(texture(lightmap, lmcoord.xy).rgb, lmcoord.y);
}