#version 150 compatibility

//#define directionalLMaps

uniform sampler2D lightmap;
uniform sampler2D normals;
uniform sampler2D specular;
uniform sampler2D texture;

flat in float blockid;
in vec2 lmcoord;
in vec2 texcoord;
flat in vec3 normal;
in vec4 viewPos;
in vec4 glcolor;
flat in mat3 tbn;

#include "/lab/common.glsl"
#include "/lab/gamma.glsl"
#include "/lab/bufferinfo.glsl"

void main() {
	vec2 coord = texcoord;
	vec2 lmapCoord = lmcoord;

	vec4 color = texture2D(texture, coord) * vec4(glcolor.rgb, 1.0);
	color.rgb = sRGBtoLinear(color.rgb);

	vec4 normalMap = texture2D(normals, coord) * 2.0 - 1.0;

	float normalAO = normalMap.z;

    normalMap.rgb = normalZRecontrust(normalMap.rgb);

	normalMap.rgb = tbn*normalMap.rgb;

	#ifdef directionalLMaps
	vec3 dFdViewposX = dFdx(viewPos.xyz);
	vec3 dFdViewposY = dFdy(viewPos.xyz);
	vec2 dFdTorch = vec2(dFdx(lmapCoord.r), dFdy(lmapCoord.r));
	
	vec3 torchLightDir = dFdViewposX * dFdTorch.x + dFdViewposY * dFdTorch.y;
	if(length(dFdTorch) > 1e-6) {
		lmapCoord.r *= clamp(dot(normalize(torchLightDir), normalMap.rgb) + 0.8, 0.0, 1.0) * 0.8 + 0.2;
	}
	else {
		lmapCoord.r *= clamp(dot(tbn * vec3(0.0, 0.0, 1.0), normal), 0.0, 1.0);
	}

	lmapCoord.rg = clamp(lmapCoord.rg, 1.0/32.0, 31.0/32.0);
	#endif

/* DRAWBUFFERS:0124*/
	gl_FragData[0] = color; //gcolor
	gl_FragData[1] = vec4(normal.xy * 0.5 + 0.5, blockid * rcp(65535.0), lmapCoord.x);
	gl_FragData[2] = vec4(normalMap.rgb * 0.5 + 0.5, normalAO);
	gl_FragData[3] = vec4(texture(lightmap, lmapCoord.xy).rgb, lmapCoord.y);
}