#version 150 compatibility

#define customWaterNormals
// #define customWaterColor

#define customWaterColorR 0.4 //[0.1 0.2 0.3 0.4 0.5 0.6 0.7 0.8 0.9 1.0]
#define customWaterColorG 0.6 //[0.1 0.2 0.3 0.4 0.5 0.6 0.7 0.8 0.9 1.0]
#define customWaterColorB 1.0 //[0.1 0.2 0.3 0.4 0.5 0.6 0.7 0.8 0.9 1.0]
#define customWaterColorA 0.1 //[0.1 0.2 0.3 0.4 0.5 0.6 0.7 0.8 0.9 1.0]

uniform sampler2D lightmap;
uniform sampler2D normals;
uniform sampler2D specular;
uniform sampler2D texture;
uniform sampler2D noisetex;

uniform vec3 cameraPosition;
uniform vec3 shadowLightPosition;

uniform float frameTimeCounter;
uniform float far;

uniform int isEyeInWater;

flat in float blockid;
in vec2 lmcoord;
in vec2 texcoord;
flat in vec3 normal;
in vec4 viewPos;
in vec3 worldPos;
in vec4 glcolor;
flat in mat3 tbn;

#include "/lab/common.glsl"
#include "/lab/gamma.glsl"
#include "/lab/bufferinfo.glsl"
#include "/lab/time.glsl"
#include "/lab/sky.glsl"
#include "/lab/fog.glsl"

void main() {
	vec2 coord = texcoord;

	bool isWater = blockid == 8.0 || blockid == 9.0;

	vec4 color = texture2D(texture, coord) * vec4(glcolor.rgb, 1.0);
	color.rgb = sRGBtoLinear(color.rgb);

	#ifdef customWaterColor
	if(isWater) color = vec4(customWaterColorR, customWaterColorG, customWaterColorB, customWaterColorA) * vec4(pow2(glcolor.rgb), 1.0);
	#endif

	vec3 lmapTexData = pow2(texture(lightmap, lmcoord).rgb);

	color.rgb *= mix(lmapTexData, byTime(vec3(1.0, 1.0, 0.4), vec3(1.0), vec3(1.0, 1.0, 0.4), vec3(0.0))*lmapTexData, lmapTexData);

	vec4 normalMap = texture2D(normals, coord) * 2.0 - 1.0;	

	float normalAO = normalMap.z;

    normalMap.rgb = normalZRecontrust(normalMap.rgb);

	#ifdef customWaterNormals
	if(isWater) normalMap.rgb = texture(noisetex, (worldPos.xz + vec2(sin(worldPos.x + frameTimeCounter) * 0.25, cos(worldPos.z + frameTimeCounter) * 0.25)) * 0.05).rgb * 2.0 - 1.0;
	#endif

	normalMap.rgb = tbn*normalMap.rgb;

	float lambertDiffuse = (max0(dot(normalMap.rgb, normalize(shadowLightPosition))));

	if(!isWater) color.rgb *= mix(vec3(0.25,0.35,0.5), vec3(1.0), lambertDiffuse);

	vec3 eyePos = worldPos - cameraPosition;
	vec3 skyData = skyCol(normalize(eyePos));

	color.rgb = fogMix(color.rgb, skyData, eyePos);

/* DRAWBUFFERS:0124 */
	gl_FragData[0] = color; //gcolor
	gl_FragData[1] = vec4(normal.xy * 0.5 + 0.5, blockid * rcp(65535.0), 1.0);
	gl_FragData[2] = vec4(normalMap.rgb * 0.5 + 0.5, normalAO);
	gl_FragData[3] = vec4(texture(lightmap, lmcoord.xy).rgb, lmcoord.y);
}